/*
 * A simple utility that allows killing toplevel X11 windows
 * using a single keypress even if they have grabbed input.
 *
 * By Kostas "Bad Sector" Michalopoulos
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/extensions/XInput2.h>

Display* dpy;
XEvent ev;
XIGrabModifiers mod;
XIEventMask mask;
int xi_opcode, i, j, error;

int error_handler(Display* d, XErrorEvent* err)
{
    (void)d; (void)err;
    error = 1;
    return 1;
}

int main()
{
    dpy = XOpenDisplay(NULL);
    if (!dpy) {
    	fprintf(stderr, "Failed to open the display, is X11 running?\n");
	return 1;
    }
    XSetErrorHandler(error_handler);
    if (!XQueryExtension(dpy, "XInputExtension", &xi_opcode, &i, &j)) {
        fprintf(stderr, "XInputExtension not found\n");
	return 1;
    }
    mask.mask_len = XIMaskLen(XI_KeyPress);
    mask.mask = calloc(1, mask.mask_len);
    XISetMask(mask.mask, XI_KeyPress);
    mod.modifiers = ControlMask|Mod1Mask; /* Ctrl+Alt */
    if (XIGrabKeycode(dpy, XIAllDevices, 45 /* K */,
        DefaultRootWindow(dpy), GrabModeAsync, GrabModeAsync,
        0, &mask, 1, &mod) != 0) {
        fprintf(stderr, "Failed to grab Ctrl+Alt+K\n");
	return 1;
    }
    free(mask.mask);
    while (1) {
    	XGenericEventCookie* cookie = &ev.xcookie;
	XNextEvent(dpy, &ev);
	if (cookie->type == GenericEvent &&
	    cookie->extension == xi_opcode) {
	    if (XGetEventData(dpy, cookie)) {
                if (cookie->evtype == 2) {
		    Window w = None;
		    error = 0;
		    XGetInputFocus(dpy, &w, &i);
		    if (w != None && !error) {
		        char tmp[64];
			sprintf(tmp, "xkill -id %lu", (long unsigned)w);
			system(tmp);
	    	    	system("sleep 0.1 ; xrandr -s 0");
		    }
		}
	    	XFreeEventData(dpy, &ev.xcookie);
	    }
	}
    }
    return 0;
}
